#
# Environment settings
#
include env.mk

#
# Use gcc-4.4 for CUDA. Use NVCC for CUDA device code
#
NVCC       := $(CUDA_INSTALL_PATH)/bin/nvcc -m64 -shared
CXX        := g++-4.4 -shared -fPIC -m64
CC         := gcc-4.4 -shared -fPIC -m64
LINK       := g++-4.4 -shared -fPIC -m64 

#
# Setup include paths
#
INCLUDES  = -I. -I$(CUDA_INSTALL_PATH)/include -I$(CUDA_DEVELOPER_PATH)/C/common/inc -I$(JDK_INCLUDE_PATH) -I$(JDK_INCLUDE_PATH)/linux -I.

#
# Use same warning settings as CUDA SDK samples
#
CXXWARN_FLAGS := \
        -W -Wall \
        -Wimplicit \
        -Wswitch \
        -Wformat \
        -Wchar-subscripts \
        -Wparentheses \
        -Wmultichar \
        -Wtrigraphs \
        -Wpointer-arith \
        -Wcast-align \
        -Wreturn-type \
        -Wno-unused-function \
        $(SPACE)
CWARN_FLAGS := $(CXXWARN_FLAGS) \
        -Wstrict-prototypes \
        -Wmissing-prototypes \
        -Wmissing-declarations \
        -Wnested-externs \
        -Wmain \
        
NVCCFLAGS	:=
LIB_ARCH	:=	x86_64

COMMONFLAGS += $(INCLUDES) -DUNIX

CXXFLAGS  := $(CXXWARN_FLAGS) $(COMMONFLAGS)
CFLAGS    := $(CWARN_FLAGS) $(COMMONFLAGS)
NVCCFLAGS += $(COMMONFLAGS)

# Debug/release configuration
ifeq ($(dbg),1)
        COMMONFLAGS += -g
        NVCCFLAGS   += -D_DEBUG --compiler-options -fno-strict-aliasing,-fPIC
        CXXFLAGS    += -D_DEBUG
        CFLAGS      += -D_DEBUG
        BINSUBDIR   := debug
        LIBSUFFIX   := D
else
        COMMONFLAGS += -O2
        BINSUBDIR   := release
        LIBSUFFIX   :=
        NVCCFLAGS   += --compiler-options -fno-strict-aliasing,-fPIC
        CXXFLAGS    += -fno-strict-aliasing
        CFLAGS      += -fno-strict-aliasing
endif

LIB       := -L$(CUDA_INSTALL_PATH)/lib64 -L$(CUDA_DEVELOPER_PATH)/C/lib -L$(CUDA_DEVELOPER_PATH)/C/Common/lib/linux -L$(JDK_LIB_PATH)/amd64 -L$(JDK_LIB_PATH)/amd64/server -L/opt/streambase/NVIDIA_GPU_Computing_SDK/lib -L/usr/local/cuda/lib64 -lcutil_x86_64 -lcuda -lcudart -ljvm

LIB_LINKLINE  =  $(LINK) -o lib$(TARGET).so -shared -Wl,-soname,lib$(TARGET).so $(OBJS) $(CUBINS) $(LIB)

 # check if verbose 
ifeq ($(verbose), 1)
        VERBOSE :=
else
        VERBOSE := @
endif

OBJDIR := $(ROOTOBJDIR)/$(LIB_ARCH)/$(BINSUBDIR)
OBJS +=  $(patsubst %.cpp,$(OBJDIR)/%.cpp.o,$(notdir $(CCFILES)))
OBJS +=  $(patsubst %.c,$(OBJDIR)/%.c.o,$(notdir $(CFILES)))
OBJS +=  $(patsubst %.cu,$(OBJDIR)/%.cu.o,$(notdir $(CUFILES)))

CUBINDIR := $(SRCDIR)data
CUBINS +=  $(patsubst %.cu,$(CUBINDIR)/%.cubin,$(notdir $(CUBINFILES)))

PTXDIR := $(SRCDIR)data
PTXBINS +=  $(patsubst %.cu,$(PTXDIR)/%.ptx,$(notdir $(PTXFILES)))

$(OBJDIR)/%.c.o : $(SRCDIR)%.c $(C_DEPS)
	$(VERBOSE)$(CC) $(CFLAGS) -o $@ -c $<

$(OBJDIR)/%.cpp.o : $(SRCDIR)%.cpp $(C_DEPS)
	$(VERBOSE)$(CXX) $(CXXFLAGS) -o $@ -c $<

$(OBJDIR)/%.cu.o : $(SRCDIR)%.cu $(CU_DEPS)
	$(VERBOSE)$(NVCC) $(NVCCFLAGS) $(SMVERSIONFLAGS) -o $@ -c $<

$(CUBINDIR)/%.cubin : $(SRCDIR)%.cu
	$(VERBOSE)$(NVCC) $(CUBIN_ARCH_FLAG) $(NVCCFLAGS) $(SMVERSIONFLAGS) -o $@ -cubin $<

$(PTXDIR)/%.ptx : $(SRCDIR)%.cu
	$(VERBOSE)$(NVCC) $(CUBIN_ARCH_FLAG) $(NVCCFLAGS) $(SMVERSIONFLAGS) -o $@ -ptx $<
        
lib$(TARGET).so: $(OBJS)
	$(VERBOSE)$(LIB_LINKLINE)

#
# Main target
#
all: mkdirs lib$(TARGET).so
