/*
 * Copyright (c) 2004-2011 StreamBase Systems, Inc.  All rights reserved.
 * Author: Darach Ennis <darach.ennis@streambase.com>
 *
 */
package com.streambase.demos.nvidia.bsm;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;

import static com.streambase.demos.nvidia.bsm.RandomUtil.*;

public class AsyncBench {
    public static void main(String args[]) {
        int max = (args.length == 0) ? Integer.MAX_VALUE : Integer.parseInt(args[0]);
        System.out.printf("BatchSize,ElapsedSec,AvgLatencySec,BatchesSec,CalcsSec,BandwidthSec:\n");
        for (int i = 1; i <= max; i = i << 1) {
            test(i);
        }
    }	

    public static void test(int optionMicroBatchSize) {
        ByteBuffer bb = ByteBuffer.allocateDirect(optionMicroBatchSize * 3 * 4);

        AsyncBsm mc = new AsyncBsm(optionMicroBatchSize);
        mc.initCuda();

        long count = 0;
        long startTime = System.currentTimeMillis();
        long startTest = 0;
        long endTest = 0;

        for(;;) {
            long now = System.currentTimeMillis();
			
            FloatBuffer fb = bb.asFloatBuffer();
            for(int i = 0; i < optionMicroBatchSize; i++) {		
                fb.put(randFloat(5.f,30.f)); // S
                fb.put(randFloat(1.f,100.f)); // X
                fb.put(randFloat(.25f,10.f)); // T
            }

            mc.send(bb);
		
            while(!mc.available()) {
                Thread.yield();
            }

//            ByteBuffer responseBuffer = mc.receive();
//            FloatBuffer fr = responseBuffer.asFloatBuffer();
		
            if (((now - startTime)/1000) < 3) continue; // warmup
            if (startTest == 0) startTest = System.currentTimeMillis();
		    
//            for (int j = 0; j < mc.getOptionMicroBatchSize(); j++) {
//                 System.out.printf("%s: a: %s b: %s c: %s put: %s, call: %s\n", j, fb.get((j*3)), fb.get((j*3)+1), fb.get((j*3)+2), fr.get((j*2)), fr.get((j*2)+1));
//            }
		    
            count++;
		    
            if (((now - startTest)/1000) > 10) {
                endTest = System.currentTimeMillis();
                break;
            }
        }

        mc.freeCuda();
		
        long elapsedMillis = (endTest - startTest);
        double elapsedSec = elapsedMillis / 1000.;
        long iterations = count;
        long calculations = iterations * optionMicroBatchSize;
        long requestHostBandwidth = calculations * 3 * 4;
        long responseHostBandwidth = calculations * 2 * 4;
        long totalHostBandwidth = requestHostBandwidth + responseHostBandwidth;
	
        double batchesSec = iterations / elapsedSec;
        double avgLatencySec = elapsedSec / iterations;
        double calculationsSec = calculations / elapsedSec;
        double totalHostBandwidthSec = totalHostBandwidth / elapsedSec / 1024 / 1024; // MB
		
        System.out.printf("%s,%s,%s,%s,%s,%s\n",
            optionMicroBatchSize,
            elapsedSec,
            avgLatencySec,
            batchesSec,
            calculationsSec,
            totalHostBandwidthSec
        );
    }
}
