/*
 * Copyright 2010 StreamBase Systems, Inc.  All rights reserved.
 * Author: Darach Ennis <darach.ennis@streambase.com>
 *
 */

package com.streambase.demos.nvidia.bsm;

import java.nio.ByteBuffer;

public class SyncBsm {
    static {
        boolean libSyncBsmIsLoaded = false;
        try {
            if (!libSyncBsmIsLoaded) {
                System.loadLibrary("SyncBsm");
                libSyncBsmIsLoaded = true;
            }
        } catch(UnsatisfiedLinkError ignoreAndContinue) {
        	// StreamBase Studio (Eclipse) may erroneously throw an ULE
        }
    }

    private long nativePtrMap;
    private float volatility = 0.06f;
    private float riskFree = .1f;
    private int optionMicroBatchSize = 100000;

    public SyncBsm(int optionMicroBatchSize) {
        this.optionMicroBatchSize = optionMicroBatchSize;
    }

    public native long nativeId();
    public native void initCuda();
    public native ByteBuffer callCuda(ByteBuffer buffer);
    public native void freeCuda();

    public int getOptionMicroBatchSize() {
        return optionMicroBatchSize;
    }

    public void setRiskFreeRate(double riskFreeRate) {
        riskFreeRate = (float)riskFreeRate; // LOSES PRECISION
    }

    public void setVolatility(double volatility2) {
        volatility = (float)volatility2; // LOSES PRECISION
    }
}
