/*
 * Copyright (c) 2004-2011 StreamBase Systems, Inc.  All rights reserved.
 * Author: Darach Ennis <darach.ennis@streambase.com>
 *
 */
package com.streambase.demos.nvidia.operator;



import java.net.URL;
import java.nio.ByteBuffer;
import java.nio.FloatBuffer;

import com.streambase.demos.nvidia.bsm.SyncBsm;
import com.streambase.sb.*;
import com.streambase.sb.operator.*;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.0.1.1012141737).
 *
 * This class is used as a Java Operator in a StreamBase application.
 * One instance will be created for each Java Operator in a StreamBase 
 * application. 
 * <p>
 * Enqueue methods should only be called from processTuple.
 * <p>
 * If there is any state that the operator instance needs to maintain beyond the
 * lifetime of the instance, the setSessionState/getSessionState methods can
 * be used.
 * @see Parameterizable
 * @see Operator
 * For in-depth information on implementing a custom Java Operator, please see
 * "Developing StreamBase Java Operators" in the StreamBase documentation.
 */
public class NvidiaCudaBlackScholes2 extends Operator implements Parameterizable {

	public static final long serialVersionUID = 1298311058003L;
	// Properties
	private int batchSize;
	private double riskFreeRate;
	private double volatility;
	private String displayName = "CUDA BSM";
	// Local variables
	private int inputPorts = 1;
	private SyncBsm nvidiaCudaSyncBsm;
	private FloatBuffer fb;
	private ByteBuffer bb;
	private Tuple pivot;

	/**
	 * The constructor is called when the Operator instance is created, but before the Operator 
	 * is connected to the StreamBase application. We recommended that you set the initial input
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount).
	 * The default is 1 input port, 1 output port. The constructor may also set default values for 
	 * operator parameters. These values will be displayed in StreamBase Studio when a new instance
	 * of this operator is  dragged to the canvas, and serve as the default values for omitted
	 * optional parameters.
	 */
	public NvidiaCudaBlackScholes2() {
		super();
		setPortHints(1, 1);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setBatchSize(100000);
		setRiskFreeRate(0.1D);
		setVolatility(0.06D);
	}

	/**
	 * The typecheck method is called after the Operator instance is connected in the StreamBase
	 * application, allowing the Operator to validate its properties. The Operator class may 
	 * change the number of input or output ports by calling the requireInputPortCount(portCount)
	 * method or the setOutputSchema(schema, portNum) method. If the verifyInputPortCount method 
	 * is passed a different number of ports than the Operator currently has, a PortMismatchException
	 * (subtype of TypecheckException) is thrown.
	 */
	public void typecheck() throws TypecheckException {
		requireInputPortCount(inputPorts);

		if (!getInputSchema(0).hasField("StockPrice")) {
			throw new TypecheckException("Input field 'StockPrice' of type list(double) is required");
		}
		if (!getInputSchema(0).hasField("OptionStrike")) {
			throw new TypecheckException("Input field 'OptionStrike' of type list(double) is required");
		}
		if (!getInputSchema(0).hasField("OptionYears")) {
			throw new TypecheckException("Input field 'OptionYears' of type list(double) is required");
		}
		try {
			if (!CompleteDataType.forList(CompleteDataType.forDouble()).equals(getInputSchema(0).getField("StockPrice").getCompleteDataType())) {
				throw new TypecheckException("Input field 'StockPrice' of type list(double) is required");
			}
			if (!CompleteDataType.forList(CompleteDataType.forDouble()).equals(getInputSchema(0).getField("OptionStrike").getCompleteDataType())) {
				throw new TypecheckException("Input field 'OptionStrike' of type list(double) is required");
			}
			if (!CompleteDataType.forList(CompleteDataType.forDouble()).equals(getInputSchema(0).getField("OptionYears").getCompleteDataType())) {
				throw new TypecheckException("Input field 'OptionYears' of type list(double) is required");
			}
		} catch (TupleException e) {
			throw new TypecheckException(e);
		}
		
		Schema outputSchema = new Schema("BsmResponseSchema", new Schema.Field[] {
			new Schema.Field("StockPrice", CompleteDataType.forDouble()),	
			new Schema.Field("OptionStrike", CompleteDataType.forDouble()),	
			new Schema.Field("OptionYears", CompleteDataType.forDouble()),	
			new Schema.Field("Put", CompleteDataType.forDouble()),	
			new Schema.Field("Call", CompleteDataType.forDouble()),	
		});
		setOutputSchema(0, outputSchema);
		
		pivot = outputSchema.createTuple();
		
		nvidiaCudaSyncBsm = new SyncBsm(getBatchSize());
		nvidiaCudaSyncBsm.setRiskFreeRate(getRiskFreeRate());
		nvidiaCudaSyncBsm.setVolatility(getVolatility());
		nvidiaCudaSyncBsm.initCuda();
	}

	/**
	 * This method will be called by the StreamBase server for each Tuple given
	 * to the Operator to process. This is the only time an operator should 
	 * enqueue output Tuples.
	 * @param inputPort the input port that the tuple is from (ports are zero based)
	 * @param pivot the tuple from the given input port
	 * @throws StreamBaseException Terminates the application.
	 */
	public void processTuple(int inputPort, Tuple tuple)
			throws StreamBaseException {
		Double[] stockPrices = tuple.getList("StockPrice").toArray(new Double[]{});
		Double[] optionStrikes = tuple.getList("OptionStrike").toArray(new Double[]{});
		Double[] optionYears = tuple.getList("OptionYears").toArray(new Double[]{});
		
		bb = ByteBuffer.allocateDirect(getBatchSize() * 4 * 3);
		fb = bb.asFloatBuffer();

		for (int i = 0; i < getBatchSize(); i++) {
			fb.put(stockPrices[i].floatValue());
			fb.put(optionStrikes[i].floatValue());
			fb.put(optionYears[i].floatValue());
		}
		
		ByteBuffer rb = nvidiaCudaSyncBsm.callCuda(bb);
		FloatBuffer rf = rb.asFloatBuffer();
		for (int i = 0; i < batchSize; i++) {
			pivot.setField("StockPrice", (double)fb.get((i*3)));
			pivot.setField("OptionStrike", (double)fb.get((i*3)+1));
			pivot.setField("OptionYears", (double)fb.get((i*3)+2));
			pivot.setField("Put", (double)rf.get((i*2)));
			pivot.setField("Call", (double)rf.get((i*2)+1));
			sendOutput(0, pivot);
			pivot.clear();
		}
		rb.clear();
		bb.clear();
		bb = null;
		rb = null;
		rf = null;
	}

	/**
	 * If typecheck succeeds, the init method is called before the StreamBase application
	 * is started. Note that your Operator class is not required to define the init method,
	 * unless (for example) you need to perform initialization of a resource such as a JDBC
	 * pool, if your operator is making JDBC calls. StreamBase Studio does not call this
	 * during authoring.
	 */
	public void init() throws StreamBaseException {
		super.init();
	}

	/**
	 *  The shutdown method is called when the StreamBase server is in the process of shutting down.
	 */
	public void shutdown() {
		nvidiaCudaSyncBsm.freeCuda();
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setBatchSize(int batchSize) {
		this.batchSize = batchSize;
	}

	public int getBatchSize() {
		return this.batchSize;
	}

	public void setRiskFreeRate(double riskFreeRate) {
		this.riskFreeRate = riskFreeRate;
	}

	public double getRiskFreeRate() {
		return this.riskFreeRate;
	}

	public void setVolatility(double volatility) {
		this.volatility = volatility;
	}

	public double getVolatility() {
		return this.volatility;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	public boolean shouldEnableBatchSize() {
		// TODO implement custom enablement logic here 
		return true;
	}

	public boolean shouldEnableRiskFreeRate() {
		// TODO implement custom enablement logic here 
		return true;
	}

	public boolean shouldEnableVolatility() {
		// TODO implement custom enablement logic here 
		return true;
	}

	@Override
	public URL getIconResource(IconKind iconType) {
		if (IconKind.CANVAS_OVERLAY_ICON.equals(iconType)) {
			return this.getClass().getResource("NVidiaCudaLogo48x48.png");
		}
		if (IconKind.PALETTE_ICON_LARGE.equals(iconType)) {
			return this.getClass().getResource("NVidiaCudaLogo24x24.png");
		}
		if (IconKind.PALETTE_ICON_SMALL.equals(iconType)) {
			return this.getClass().getResource("NVidiaCudaLogo16x16.png");
		}
		return super.getIconResource(iconType);
	}

	
}
