/*
 * Copyright (c) 2004-2011 StreamBase Systems, Inc.  All rights reserved.
 * Author: Darach Ennis <darach.ennis@streambase.com>
 *
 */
package com.streambase.demos.nvidia.operator;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.util.ArrayList;
import java.util.List;

import com.streambase.demos.nvidia.bsm.SyncBsm;
import com.streambase.sb.CompleteDataType;
import com.streambase.sb.NullValueException;
import com.streambase.sb.Schema;
import com.streambase.sb.StreamBaseRuntimeException;
import com.streambase.sb.Tuple;
import com.streambase.sb.TupleException;
import com.streambase.sb.client.CustomFunctionResolver;
import com.streambase.sb.operator.AggregateWindow;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.0.1.1012141737).
 *
 * All custom java aggregate classes must extend AggregateWindow. The user's subclass
 * must implement an accumulate method that corresponds to the argument number and
 * data types that the calljava expression in the Aggregate operator refers to. 
 * The user's subclass must also implement a calculate method. When data is sent from 
 * the aggregate window, the calculate method must return a type mapping to a StreamBase data 
 * type: int, Integer, double, Double, Timestamp, byte [], or String. 
 * The AggregateWindow object is likely to be used several times. At the start of each 
 * new use, the init() method will be called. Then accumulate and calculate may be called.
 * Finally, at the end of each use, release will be called. 
 * They may also be aliased for use as if that function were embedded in 
 * your StreamBase application by using the alias attribute within the <custom-functions> section
 * of the StreamBase Server configuration file.
 * <p>
 * For in-depth information on implementing a a StreamBase aggregate function, please see some
 * related topics from "Developing StreamBase Custom Functions" in the StreamBase 
 * documentation.
 * For more information on using aliases, please see the <custom-functions> section from 
 * "StreamBase Server Configuration XML" in the StreamBase documentation.
 * <p>
 */
public class NvidiaCudaBsmFn extends AggregateWindow {

	public static final long serialVersionUID = 1298322835490L;
	private SyncBsm nvidiaCudaSyncBsm;
	private ByteBuffer bb;
	private FloatBuffer fb;
	private int count;

	/**
	 * Called before each new use of the aggregate window object.
	 */
	public void init() {
		count = 0;
	}

	/**
	 * The calculate method will be called whenever a new tuple is generated.
	 * The calculate method may be called just once (in the case when a tuple is
	 * generated when the aggregate window closes) or many times (in the case when
	 * the aggregate window remains open, but a tuple is generated because "emit every 
	 * N tuples" was selected). It is possible that calculate may be called before
	 * accumulate is called. It is also possible that calculate may never be called.  
	 */
	public Tuple calculate() {
		ByteBuffer rb = nvidiaCudaSyncBsm.callCuda(bb);
		FloatBuffer rf = rb.asFloatBuffer();
		
		Tuple pivot = RESULT_SCHEMA.createTuple();
		List<Double> puts = new ArrayList<Double>();
		List<Double> calls = new ArrayList<Double>();
		for (int i = 0; i < count; i++) {
			puts.add(i, (double)rf.get((i*2)));
			calls.add(i, (double)rf.get((i*2)+1));
		}
		try {
			pivot.setList("Puts", puts);
			pivot.setList("Calls", calls);
		} catch (NullValueException e) {
			throw new StreamBaseRuntimeException(e);
		} catch (TupleException e) {
			throw new StreamBaseRuntimeException(e);
		}
		return pivot;
	}
	
	private static final Schema.Field[] RESULT_FIELDS = new Schema.Field[] {
			new Schema.Field("Puts", CompleteDataType.forList(CompleteDataType.forDouble())),
			new Schema.Field("Calls", CompleteDataType.forList(CompleteDataType.forDouble())),
	};
	private static final Schema RESULT_SCHEMA = new Schema("NvidiaCudaAggFnSchema", RESULT_FIELDS);
	
	public static CompleteDataType resolver(CompleteDataType s, CompleteDataType x, CompleteDataType y, CompleteDataType b, CompleteDataType r, CompleteDataType v) {
		return CompleteDataType.forTuple(RESULT_SCHEMA);
	}

	/**
	 * The accumulate method will be called each time new data arrives for the window.
	 * There may be many overloaded accumulate methods. calljava determines which one
	 * to call based on type. All accumulate methods must have a void return. There must be
	 * at least one accumulate method.
	 * The function can be assigned an alias. It can then be called using the alias name instead of
	 * using calljava().
	 */
	@CustomFunctionResolver("resolver")
	public void accumulate(double StockPrice, double OptionStrike,
			double OptionYears, int BatchSize, double RiskFreeRate,
			double Volatility) {
		if (nvidiaCudaSyncBsm == null) {
			nvidiaCudaSyncBsm = new SyncBsm(BatchSize);
			nvidiaCudaSyncBsm.setRiskFreeRate(RiskFreeRate);
			nvidiaCudaSyncBsm.setVolatility(Volatility);
			nvidiaCudaSyncBsm.initCuda();
		}
		
		if (fb == null) {
			bb = ByteBuffer.allocateDirect(BatchSize * 4 * 3);
			fb = bb.asFloatBuffer();
		}

		fb.put((float)StockPrice);
		fb.put((float)OptionStrike);
		fb.put((float)OptionYears);
		count++;
	}

	/**
	 * Called after the end of a use of the aggregate window object.
	 */
	public void release() {
		bb.clear();
		bb = null;
		fb = null;
//		nvidiaCudaSyncBsm.freeCuda();
	}

}