/*
 * Copyright (c) 2004-2011 StreamBase Systems, Inc.  All rights reserved.
 * Author: Darach Ennis <darach.ennis@streambase.com>
 *
 */
package com.streambase.demos.nvidia.operator;



import java.util.LinkedList;
import java.util.List;

import com.streambase.sb.*;
import com.streambase.sb.adapter.*;
import com.streambase.sb.operator.*;

import static com.streambase.demos.nvidia.bsm.RandomUtil.*;

/**
 * Generated by JDT StreamBase Client Templates (Version: 7.0.1.1012141737).
 *
 * Input adapters take information from outside of an application and
 * provide it in the form of a stream or streams to the application. 
 * Any implementation of this class <b>must</b> extend class InputAdapter.
 * Input adapters should call sendOutput(int, Tuple) to provide tuples 
 * to the application.
 * <p>
 * For in-depth information on implementing a custom adapter, please
 * see "Developing StreamBase Embedded Adapters" in the StreamBase documentation.
 * <p>
 */
public class PriceBlaster extends InputAdapter implements Parameterizable,
		Runnable {

	public static final long serialVersionUID = 1298326651692L;
	// Properties
	private int batchSize;
	private Schema schema0;
	private String displayName = "Price Blaster";
	private Blaster blaster;

	/**
	 * The constructor is called when the Adapter instance is created, but before the Adapter 
	 * is connected to the StreamBase application. We recommended that you set the initial input 
	 * port and output port count in the constructor by calling setPortHints(inPortCount, outPortCount). 
	 * The default is no input ports or output ports. The constructor may also set default values for 
	 * adapter parameters. These values will be displayed in StreamBase Studio when a new adapter is 
	 * dragged to the canvas, and serve as the default values for omitted optional parameters.
	 */
	public PriceBlaster() {
		super();
		setPortHints(0, 1);
		setDisplayName(displayName);
		setShortDisplayName(this.getClass().getSimpleName());
		setBatchSize(100000);
	}
	
	private static final Schema.Field[] SINGLE_FIELDS = new Schema.Field[] {
		new Schema.Field("Symbol", CompleteDataType.forString()),
		new Schema.Field("StockPrice", CompleteDataType.forDouble()),
		new Schema.Field("OptionStrike", CompleteDataType.forDouble()),
		new Schema.Field("OptionYears", CompleteDataType.forDouble()),
	};

	private static final Schema SINGLE_SCHEMA = new Schema("SingleSchema", SINGLE_FIELDS);

	private static final Schema.Field[] MULTI_FIELDS = new Schema.Field[] {
		new Schema.Field("Data", CompleteDataType.forList(CompleteDataType.forTuple(SINGLE_SCHEMA)))
	};

	private static final Schema MULTI_SCHEMA = new Schema("MultiSchema", MULTI_FIELDS);
	
	/**
	 * Typecheck this adapter. The typecheck method is called after the adapter instance is
	 * connected in the StreamBase application. The adapter should validate its parameters 
	 * and throw TypecheckException if any problems are found. The message associated with 
	 * the thrown TypecheckException will be displayed in StreamBase Studio during authoring,
	 * or printed on the console by sbd. Input adapters should set the schema of each output
	 * port by calling the setOutputSchema(portNum, schema) method for each output port. 
	 * If the adapter needs to change the number of input ports based on parameter values,
	 * it should call requireInputPortCount(portCount) at this point. 
	 */
	public void typecheck() throws TypecheckException {
		setOutputSchema(0, getBatchSize() == 0 ? SINGLE_SCHEMA : MULTI_SCHEMA);
	}

	private static interface Blaster {
		public Tuple blast();
	}
	
	public class SingleBlaster implements Blaster {
		Tuple pivot = SINGLE_SCHEMA.createTuple();
		public Tuple blast() {
			pivot.clear();
			try {
				pivot.setDouble("StockPrice", (double)randFloat(5.f, 30.f));
				pivot.setDouble("OptionStrike", (double)randFloat(10.f, 30.f));
				pivot.setDouble("OptionYears", (double)randFloat(.1f, 10.f));
			} catch (NullValueException e) {
				throw new StreamBaseRuntimeException(e);
			} catch (TupleException e) {
				throw new StreamBaseRuntimeException(e);
			}
			return pivot;
		}		
	}
	
	public class MultiBlaster implements Blaster {
		Tuple result = MULTI_SCHEMA.createTuple();
		Tuple pivot = SINGLE_SCHEMA.createTuple();
		List<Tuple> list = new LinkedList<Tuple>();
		int count = 0;
		public Tuple blast() {
			pivot.clear();
			list.clear();
			try {
				pivot.setString("Symbol", "FOO" + count++);
				pivot.setDouble("StockPrice", (double)randFloat(5.f, 30.f));
				pivot.setDouble("OptionStrike", (double)randFloat(10.f, 30.f));
				pivot.setDouble("OptionYears", (double)randFloat(.1f, 10.f));
				list.add(pivot.clone());
				result.setList(0, list);
			} catch (NullValueException e) {
				throw new StreamBaseRuntimeException(e);
			} catch (TupleException e) {
				throw new StreamBaseRuntimeException(e);
			} catch (CloneNotSupportedException e) {
				throw new StreamBaseRuntimeException(e);
			}
			return result;
		}		
	}

	/**
	 * Initialize the adapter. If typecheck succeeds, the init method is called before
	 * the StreamBase application is started. Note that your adapter is not required to
	 * define the init method, unless you need to register a runnable or perform
	 * initialization of a resource such as, for example, a JDBC pool.
	 */
	public void init() throws StreamBaseException {
		super.init();
		blaster = getBatchSize() == 0 ? new SingleBlaster() : new MultiBlaster();
		registerRunnable(this, true);
	}

	/**
	 * Shutdown adapter by cleaning up any resources used (e.g. close an output file
	 * if it has been opened). When the application is shutting down, the adapter's shutdown.
	 * method will be called first. Once this has returned, all threads should exit and the
	 * adapter is considered shutdown.
	 */
	public void shutdown() {

	}

	/**
	 * Main thread of the adapter. At this point, the application begins to run.
	 * StreamBase will start threads for any managed runnables registered by
	 * earlier calls to registerRunnable. Input adapters can call sendOutput(port, tuple) 
	 * at any time to output a tuple to the specified port.
	 */
	public void run() {
		// fill the tuple and output the tuple
		fillAndOutputTuple();
	}

	protected void fillAndOutputTuple() {
		try {
			while (shouldRun()) {
				try {
					sendOutput(0, blaster.blast());
				} catch (StreamBaseException e) {
					e.printStackTrace();
				}
				Thread.yield();
			}
		} finally {
			try {
				// TODO Be sure to close the resource
			} catch (Exception e) {
			}
		}
	}

	/***************************************************************************************
	 * The getter and setter methods provided by the Parameterizable object.               *
	 * StreamBase Studio uses them to determine the name and type of each property         *
	 * and obviously, to set and get the property values.                                  *
	 ***************************************************************************************/

	public void setBatchSize(int batchSize) {
		this.batchSize = batchSize;
	}

	public int getBatchSize() {
		return this.batchSize;
	}

	public Schema getSchema0() {
		return this.schema0;
	}

	public void setSchema0(Schema schema0) {
		this.schema0 = schema0;
	}

	/** For detailed information about shouldEnable methods, see interface Parameterizable java doc 
	 *  @see Parameterizable 
	 */

	public boolean shouldEnableBatchSize() {
		// TODO implement custom enablement logic here 
		return true;
	}

}
